document.addEventListener('DOMContentLoaded', function() {
    const saveButton = document.getElementById('saveRiskAssessment');
    const projectId = getProjectIdFromUrl();

    if (saveButton) {
        saveButton.addEventListener('click', saveRiskAssessmentData);
    } else {
        console.error('Save button not found');
    }

    if (projectId) {
        // Fetch compliance data first
        fetchComplianceData().then(complianceData => {
            Promise.all([
                fetchNonConformities(projectId),
                fetchRiskAssessmentData(projectId)
            ]).then(([nonConformities, riskAssessmentData]) => {
                // Pass complianceData as an argument here
                mergeAndPopulateTable(nonConformities, riskAssessmentData, complianceData);
                fetchProjectDetails(projectId);
            }).catch(error => {
                console.error('Error fetching project-related data:', error);
            });
        }).catch(error => {
            console.error('Error fetching compliance data:', error);
        });
    } else {
        console.error('Project ID is missing');
    }
});

function getProjectIdFromUrl() {
    const pathArray = window.location.pathname.split('/');
    const projectIdIndex = pathArray.findIndex(element => element === 'risk-assessment-treatment') + 1;
    return pathArray[projectIdIndex];
}

function fetchProjectDetails(projectId) {
    fetch(`/fetch-ra-rtp-data?projectId=${projectId}`)
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        })
        .then(projectDetails => {
            updateProjectDetailsTable(projectDetails);
        })
        .catch(error => {
            console.error('There has been a problem with your fetch operation:', error);
        });
}

function fetchRiskAssessmentData(projectId) {
    return fetch(`/api/fetch-risk-assessment-data?projectId=${projectId}`)
        .then(response => {
            console.log('Risk Assessment response:', response);
            return response.json();
        })
        .then(data => {
            console.log('Risk Assessment data:', data);
            return data; // Ensure data is returned for further processing
        })
        .catch(error => console.error('Error fetching risk assessment data:', error));
}

function fetchNonConformities(projectId) {
    return fetch(`/api/non-conformities/${projectId}`)
        .then(response => {
            console.log('Non-conformities response:', response);
            return response.json();
        })
        .then(data => {
            console.log('Non-conformities data:', data);
            return data; // Ensure data is returned for further processing
        })
        .catch(error => {
            console.error('Error fetching non-conformities:', error);
        });
}

async function fetchComplianceData() {
    try {
        const response = await fetch('/api/compliance-data');
        if (!response.ok) {
            throw new Error('Failed to fetch compliance data');
        }
        return await response.json();
    } catch (error) {
        console.error('Error fetching compliance data:', error);
        throw error; // Re-throw to handle it in the calling context
    }
}

function mergeAndPopulateTable(nonConformities, riskAssessmentData, complianceData) {
    // Ensure complianceData is received as a parameter
    const tableBody = document.querySelector('.scrollable-table tbody');
    tableBody.innerHTML = '';

    const dataToUse = riskAssessmentData.length > 0 ? riskAssessmentData : nonConformities;

    dataToUse.forEach((item, index) => {
        const row = tableBody.insertRow();
        const assessment = riskAssessmentData.length > 0 ? item : undefined;
        const nonConformity = nonConformities.length > index ? nonConformities[index] : undefined;

        // Pass complianceData to populateRow
        populateRow(row, index + 1, assessment, nonConformity, complianceData);
    });
}


function populateRow(row, no, assessment, nonConformity, complianceData) {
console.log('Risk Owner from Assessment:', assessment ? assessment.riskOwner : 'No assessment');
    // No. column
    const noCell = row.insertCell();
    noCell.textContent = no;

    // Name of Asset/Threat-based Analysis dropdown
    const assetCell = row.insertCell();
    const assetDropdown = createAssetDropdown(assessment.asset_name);
    assetCell.appendChild(assetDropdown);

    // Risk Owner dropdown
    const ownerCell = row.insertCell();
    const riskOwnerValue = assessment ? assessment.risk_owner : '';
    const ownerDropdown = createRiskOwnerDropdown(riskOwnerValue);
    ownerCell.appendChild(ownerDropdown);

    // Threat dropdown
    const threatCell = row.insertCell();
    const threatDropdown = createThreatDropdown(assessment ? assessment.threat : '');
    threatCell.appendChild(threatDropdown);

    // Vulnerability from non-conformity or assessment
    const vulnerabilityCell = row.insertCell();
    vulnerabilityCell.textContent = nonConformity ? nonConformity['Non-Conformity'] : (assessment ? assessment.vulnerability : '');

    // Impact input
    const impactCell = row.insertCell();
    const impactInput = createNumericInput(1, 5, assessment ? assessment.impact : '');
    impactCell.appendChild(impactInput);

    // Probability input
    const probabilityCell = row.insertCell();
    const probabilityInput = createNumericInput(1, 5, assessment ? assessment.probability : '');
    probabilityCell.appendChild(probabilityInput);

    // Calculated Risk
    const riskCell = row.insertCell();
    riskCell.textContent = assessment ? calculateRisk(assessment.impact, assessment.probability) : '';

    // Risk Treatment Options dropdown
    const treatmentOptionsCell = row.insertCell();
    const treatmentOptionsDropdown = createTreatmentOptionsDropdown(assessment ? assessment.treatment_options : '');
    treatmentOptionsCell.appendChild(treatmentOptionsDropdown);

    // Security Controls dropdown
    const securityControlsDropdown = createSecurityControlsDropdown(assessment ? assessment.security_controls : '', complianceData);
    const securityControlsCell = row.insertCell();
    securityControlsCell.appendChild(securityControlsDropdown);


    // Implementation Technique input
    const implementationCell = row.insertCell();
    const implementationInput = document.createElement('input');
    implementationInput.type = 'text';
    implementationInput.value = assessment ? assessment.implementationTechnique : '';
    implementationCell.appendChild(implementationInput);

    // Due Date dropdown
    const dueDateCell = row.insertCell();
    const dueDateDropdown = createDropdown(['Q1-2024', 'Q2-2024', 'Q3-2024', 'Q4-2024'], assessment ? assessment.dueDate : '');
    dueDateCell.appendChild(dueDateDropdown);

    // Impact After Treatment input, initially matching the Impact input
    const impactAfterTreatmentCell = row.insertCell();
    const impactAfterTreatmentInput = createNumericInput(1, 5, impactInput.value); // Use impactInput.value for initial value
    impactAfterTreatmentCell.appendChild(impactAfterTreatmentInput);

    // Probability After Treatment input
    const probabilityAfterTreatmentCell = row.insertCell(); // Ensure this cell is defined
    const probabilityAfterTreatmentInput = createNumericInput(1, 5, assessment ? assessment.probabilityAfterTreatment : '');
    probabilityAfterTreatmentCell.appendChild(probabilityAfterTreatmentInput);

    // Target Residual Risk cell
    const targetResidualRiskCell = row.insertCell();
    targetResidualRiskCell.textContent = assessment ? calculateTargetResidualRisk(impactAfterTreatmentInput.value, probabilityAfterTreatmentInput.value) : '';

    impactInput.addEventListener('change', () => {
        impactAfterTreatmentInput.value = impactInput.value;
        // Recalculate Target Residual Risk when Impact changes
        targetResidualRiskCell.textContent = calculateTargetResidualRisk(impactAfterTreatmentInput.value, probabilityAfterTreatmentInput.value);
    });

    // Always recalculate Target Residual Risk when Impact After Treatment or Probability After Treatment changes
    impactAfterTreatmentInput.addEventListener('change', () => {
        targetResidualRiskCell.textContent = calculateTargetResidualRisk(impactAfterTreatmentInput.value, probabilityAfterTreatmentInput.value);
    });

    probabilityAfterTreatmentInput.addEventListener('change', () => {
        targetResidualRiskCell.textContent = calculateTargetResidualRisk(impactAfterTreatmentInput.value, probabilityAfterTreatmentInput.value);
    });

    // Hours input
    const hoursCell = row.insertCell();
    const hoursInput = createNumericInput(0, 1000, assessment ? assessment.hours : '');
    hoursCell.appendChild(hoursInput);
}

function updateProjectDetailsTable(data) {
    const projectNameCell = document.querySelector('#project-details tr:nth-child(1) td');
    const dateOfSubmissionCell = document.querySelector('#project-details tr:nth-child(2) td');
    const createdByCell = document.querySelector('#project-details tr:nth-child(3) td');
    const versionNumberCell = document.querySelector('#project-details tr:nth-child(4) td'); // Changed variable name to versionNumberCell for clarity
    const customerNameCell = document.querySelector('#project-details tr:nth-child(6) td');

    if (projectNameCell) projectNameCell.textContent = data.project_name;
    if (dateOfSubmissionCell) dateOfSubmissionCell.textContent = data.date_of_submission;
    if (createdByCell) createdByCell.textContent = data.created_by;
    if (versionNumberCell) versionNumberCell.textContent = data.version_number; // Use data.version_number to set the text content
    if (customerNameCell) customerNameCell.textContent = data.customer_name;
}

function saveRiskAssessmentData() {
    const projectId = getProjectIdFromUrl(); // Assuming this function returns the current projectId
    const tableRows = document.querySelectorAll('.scrollable-table tbody tr');
    const riskAssessmentData = {
        projectId: projectId,
        risks: []
    };

    tableRows.forEach(row => {
        // Find the asset name field, which could be a 'select' or an 'input' if the user has chosen 'Free Text'
        const assetField = row.querySelector('td:nth-child(2) select, td:nth-child(2) input');
        const assetName = assetField.tagName.toLowerCase() === 'select' ? assetField.value : assetField.value; // Use value directly for both select and input

        const rowData = {
            no: row.querySelector('td:nth-child(1)').textContent,
            assetName: assetName,
            riskOwner: row.querySelector('td:nth-child(3) select').value,
            threat: row.querySelector('td:nth-child(4) select').value,
            vulnerability: row.querySelector('td:nth-child(5)').textContent,
            impact: row.querySelector('td:nth-child(6) input').value,
            probability: row.querySelector('td:nth-child(7) input').value,
            risk: row.querySelector('td:nth-child(8)').textContent,
            treatmentOptions: row.querySelector('td:nth-child(9) select').value,
            securityControls: row.querySelector('td:nth-child(10) select').value,
            implementationTechnique: row.querySelector('td:nth-child(11) input').value,
            dueDate: row.querySelector('td:nth-child(12) select').value,
            impactAfterTreatment: row.querySelector('td:nth-child(13) input').value,
            probabilityAfterTreatment: row.querySelector('td:nth-child(14) input').value,
            targetResidualRisk: row.querySelector('td:nth-child(15)').textContent,
            hours: row.querySelector('td:nth-child(16) input').value
        };
        riskAssessmentData.risks.push(rowData);
    });

    // Now send this data to the server
    fetch('/api/save-risk-assessment', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify(riskAssessmentData)
    })
    .then(response => {
        if (!response.ok) {
            throw new Error('Network response was not ok');
        }
        return response.json();
    })
    .then(data => {
        console.log('Save successful:', data);
        // Implement any success message display or further actions here
    })
    .catch(error => {
        console.error('Error saving risk assessment data:', error);
        // Implement any error handling logic here
    });
}

// DropDowns Functions //
function createAssetDropdown(selectedValue) {
    console.log("Selected Value at start: ", selectedValue); // Log the initial selected value

    const dropdown = document.createElement('select');
    const assetOptions = [
        'Corporate Security', 'Endpoint Security', 'Encryption Security',
        'Network Security', 'Production Security', 'Product Security',
        'Physical Security', 'Application Security', 'Data Security',
        'Cloud Security', 'Mobile Security', 'Identity Management Security',
        'Vulnerability Assessment', 'Compliance Assessment', 'Incident Response',
        'Insider Threat Detection', 'Free Text'
    ];

    assetOptions.forEach(option => {
        const optionElement = document.createElement('option');
        optionElement.value = option;
        optionElement.textContent = option;
        dropdown.appendChild(optionElement);

        if (option === selectedValue) {
            console.log(`Matching option found for: ${selectedValue}`); // Log when a match is found
            optionElement.selected = true;
        }
    });

    // Log the final selected value of the dropdown
    dropdown.addEventListener('change', () => console.log("Dropdown changed to: ", dropdown.value));

    // Handling 'Free Text' option
    if (!assetOptions.includes(selectedValue) && selectedValue) {
        console.log(`Adding 'Free Text' option for: ${selectedValue}`); // Log the addition of 'Free Text' option
        const freeTextOption = document.createElement('option');
        freeTextOption.value = selectedValue;
        freeTextOption.textContent = selectedValue;
        freeTextOption.selected = true;
        dropdown.prepend(freeTextOption); // Add 'Free Text' option to the top
    }

    return dropdown;
}

function createNumericInput(min, max) {
    const input = document.createElement('input');
    input.type = 'number';
    input.min = min;
    input.max = max;
    return input;
}

function updateTargetResidualRiskCalculation(impactInput, probabilityInput, targetRiskCell) {
    const impactValue = parseInt(impactInput.value) || 0;
    const probabilityValue = parseInt(probabilityInput.value) || 0;
    const targetRiskValue = impactValue * probabilityValue; // Target Residual Risk calculation
    targetRiskCell.textContent = targetRiskValue.toString(); // Update the cell content
}

function updateRiskCalculation(impactInput, probabilityInput, riskCell) {
    const impactValue = parseInt(impactInput.value) || 0;
    const probabilityValue = parseInt(probabilityInput.value) || 0;
    const riskValue = impactValue * probabilityValue; // Risk calculation
    riskCell.textContent = riskValue.toString(); // Update the cell content
}

function createDropdown(options, selectedValue) {
    const dropdown = document.createElement('select');
    options.forEach(option => {
        const optionElement = document.createElement('option');
        optionElement.value = option;
        optionElement.textContent = option;
        if (option === selectedValue) {
            optionElement.selected = true;
        }
        dropdown.appendChild(optionElement);
    });
    return dropdown;
}

function createNumericInput(min, max, value) {
    const input = document.createElement('input');
    input.type = 'number';
    input.min = min;
    input.max = max;
    input.value = value || min; // Default to min if no value is provided
    return input;
}

function calculateRisk(impact, probability) {
    return (parseInt(impact) || 0) * (parseInt(probability) || 0);
}

// Function to calculate Target Residual Risk
function calculateTargetResidualRisk(impactAfterTreatment, probabilityAfterTreatment) {
    return (parseInt(impactAfterTreatment) || 0) * (parseInt(probabilityAfterTreatment) || 0);
}

function updateRisk(impactInput, probabilityInput, riskCell) {
    const impactValue = parseInt(impactInput.value) || 0;
    const probabilityValue = parseInt(probabilityInput.value) || 0;
    const riskValue = impactValue * probabilityValue;
    riskCell.textContent = riskValue.toString();
}

function updateTargetResidualRisk(impactInput, probabilityInput, targetRiskCell) {
    const impactValue = parseInt(impactInput.value) || 0;
    const probabilityValue = parseInt(probabilityInput.value) || 0;
    const targetRiskValue = impactValue * probabilityValue; // Target Residual Risk calculation
    targetRiskCell.textContent = targetRiskValue.toString(); // Update the cell content
}

function createRiskOwnerDropdown(selectedValue) {
    console.log('Selected Risk Owner:', selectedValue); // Log to check what's being passed as the selected value

    const dropdown = document.createElement('select');
    const riskOwnerOptions = [
        'CEO', 'COO', 'CISO', 'CTO', 'CFO', 'VP R&D',
        'General Counsel', 'Legal Officer', 'CMO', 'CHRO',
        'CSO', 'VP H&R', 'DPO', 'Free Text'
    ];

    riskOwnerOptions.forEach(option => {
        const optionElement = document.createElement('option');
        optionElement.value = option;
        optionElement.textContent = option;
        if (option === selectedValue) {
            optionElement.selected = true;
        }
        dropdown.appendChild(optionElement);
    });

    // Handle free text if the selected value is not in the predefined list
    if (!riskOwnerOptions.includes(selectedValue) && selectedValue) {
        const freeTextOption = document.createElement('option');
        freeTextOption.value = selectedValue;
        freeTextOption.textContent = selectedValue;
        freeTextOption.selected = true;
        dropdown.appendChild(freeTextOption);
    }

    return dropdown;
}

function createThreatDropdown(selectedValue) {
    const dropdown = document.createElement('select');
    const threatOptions = [
        'Data leakage', 'Data theft', 'Malware infection', 'Unauthorized access',
        'Unauthorized disclosure', 'Availability', 'Denial of Service (DoS)',
        'Ransomware', 'Social engineering', 'Credential theft', 'Free Text'
    ];

    threatOptions.forEach(option => {
        const optionElement = document.createElement('option');
        optionElement.value = option;
        optionElement.textContent = option;
        if (option === selectedValue) {
            optionElement.selected = true;
        }
        dropdown.appendChild(optionElement);
    });

    // Handle free text if the selected value is not in the predefined list
    if (!threatOptions.includes(selectedValue) && selectedValue) {
        const freeTextOption = document.createElement('option');
        freeTextOption.value = selectedValue;
        freeTextOption.textContent = selectedValue;
        freeTextOption.selected = true;
        dropdown.appendChild(freeTextOption);
    }

    return dropdown;
}

function createTreatmentOptionsDropdown(selectedValue) {
    const dropdown = document.createElement('select');
    const treatmentOptions = [
        'Selection of controls', 'Transfer of risks to a third party',
        'Risk avoidance', 'Risk acceptance', 'Risk Mitigation', 'Free Text'
    ];

    treatmentOptions.forEach(option => {
        const optionElement = document.createElement('option');
        optionElement.value = option;
        optionElement.textContent = option;
        if (option === selectedValue) {
            optionElement.selected = true;
        }
        dropdown.appendChild(optionElement);
    });

    // Handle free text if the selected value is not in the predefined list
    if (!treatmentOptions.includes(selectedValue) && selectedValue) {
        const freeTextOption = document.createElement('option');
        freeTextOption.value = selectedValue;
        freeTextOption.textContent = selectedValue;
        freeTextOption.selected = true;
        dropdown.appendChild(freeTextOption);
    }

    return dropdown;
}

function createSecurityControlsDropdown(selectedValue, complianceData) {
    const dropdown = document.createElement('select');
    // Assuming complianceData is an array of objects with 'control_id' and 'control_text' properties
    complianceData.forEach(dataItem => {
        const option = document.createElement('option');
        option.value = `${dataItem.control_id}`;
        option.textContent = `${dataItem.control_id} ${dataItem.control_text}`;
        if (option.value === selectedValue) {
            option.selected = true;
        }
        dropdown.appendChild(option);
    });

    // Add 'Free Text' option
    const freeTextOption = document.createElement('option');
    freeTextOption.value = 'Free Text';
    freeTextOption.textContent = 'Free Text';
    if ('Free Text' === selectedValue) {
        freeTextOption.selected = true;
    }
    dropdown.appendChild(freeTextOption);

    return dropdown;
}

